//------------------------------------------------------------------------ 2012-06-19 ----------
// XebsO[^[vO 
//
//  PIC18F4550  Clock 48MHz (20MHz crystal)
//-------------------------------------------------------- Written by Hiroyuki Iizuka ----------
#include <p18f4550.h>

#include <delays.h>
#include <stdio.h>
#include <stdlib.h>
#include <EEP.h>
#include <stddef.h>

#define VERSION  "Ver1.00"

//============@u[g[_[gȂ͂Rgɂ ==========================
// #define USE_BOOTLOADER
//========================================================================

//------------------- RtBO[V[h̐ݒ ----------------------------------------
// u[g[_[gpƂ̓u[g[_[̃RtBO[V[hgp̂ŁA̐ݒ͖
#pragma config USBDIV  = 2
#pragma config PLLDIV  = 5
#pragma config CPUDIV  = OSC1_PLL2
#pragma config FOSC    = HSPLL_HS
#pragma config PWRT    = ON         // Power-up Timer
#pragma config BOR     = ON
#pragma config BORV    = 3          // Brown-out Reset Voltage Minimum setting
#pragma config WDT     = OFF        // WDT disabled (control is placed on the SWDTEN bit)
#pragma config MCLRE   = OFF        // RE3 input pin enabled; MCLR pin disabled
#pragma config LVP     = OFF
#pragma config PBADEN  = OFF

#pragma config FCMEN   = OFF        // Fail-Safe Clock Monitor disabled
#pragma config VREGEN  = OFF        // USB voltage regulator disabled

#pragma config CP0     = OFF        // Block 0 (000800-001FFFh) is not code-protected
#pragma config CP1     = OFF        // Block 1 (002000-003FFFh) is not code-protected
#pragma config CP2     = OFF        // Block 2 (004000-005FFFh) is not code-protected
#pragma config CP3     = OFF        // Block 3 (006000-007FFFh) is not code-protected
#pragma config CPB     = OFF        // Boot block (000000-0007FFh) is not code-protected
#pragma config CPD     = OFF        // Data EEPROM is not code-protected
#pragma config WRT0    = OFF        // Block 0 (000800-001FFFh) is not write-protected
#pragma config WRT1    = OFF        // Block 1 (002000-003FFFh) is not write-protected
#pragma config WRT2    = OFF        // Block 2 (004000-005FFFh) is not write-protected
#pragma config WRT3    = OFF        // Block 3 (006000-007FFFh) is not write-protected
#pragma config WRTC    = OFF        // Configuration registers (300000-3000FFh) are not write-protected
#pragma config WRTB    = OFF        // Boot block (000000-0007FFh) is not write-protected
#pragma config WRTD    = OFF        // Data EEPROM is not write-protected
#pragma config EBTR0   = OFF        // Block 0 (000800-001FFFh) is not protected from table reads executed in other blocks
#pragma config EBTR1   = OFF        // Block 1 (002000-003FFFh) is not protected from table reads executed in other blocks 
#pragma config EBTR2   = OFF        // Block 2 (004000-005FFFh) is not protected from table reads executed in other blocks
#pragma config EBTR3   = OFF        // Block 3 (006000-007FFFh) is not protected from table reads executed in other blocks
#pragma config EBTRB   = OFF        // Boot block (000000-0007FFh) is not protected from table reads executed in other blocks

//----- ʓIȒ` --------------------------------------------------------------
#define PORT_OUTPUT   0
#define PORT_INPUT    1

typedef unsigned char boolean; 
#define true  1
#define false 0

#define stop(void)    while(1) { ; }

//---------------- h^n|[g` -------------------------------------------------------------
//----- t --------------------------------------------------------------------------------
#define LCD_RS         LATAbits.LATA4
#define LCD_E          LATAbits.LATA5
#define LCD_DB4        LATAbits.LATA0
#define LCD_DB5        LATAbits.LATA1
#define LCD_DB6        LATAbits.LATA2
#define LCD_DB7        LATAbits.LATA3

#define LCD_RS_TRIS    TRISAbits.RA4
#define LCD_E_TRIS     TRISAbits.RA5
#define LCD_DB4_TRIS   TRISAbits.RA0
#define LCD_DB5_TRIS   TRISAbits.RA1
#define LCD_DB6_TRIS   TRISAbits.RA2
#define LCD_DB7_TRIS   TRISAbits.RA3

enum type_LCD_cursor { CURSOR_ON , CURSOR_OFF , BLINK_ON , BLINK_OFF };

void lcd_clear(void);
void lcd_enable_delay_us(unsigned short);
void lcd_init(void);
void lcd_cursor( enum type_LCD_cursor,  enum type_LCD_cursor );
void lcd_locate( unsigned char , unsigned char );
void lcd_const_string( const rom char * );

//------------ TB6560AHQ[^[hCo[ --------------------------------------------------------
#define MOTOR_CLOCK        LATBbits.LATB1   
#define MOTOR_RESET        LATBbits.LATB2   
#define MOTOR_ENABLE       LATBbits.LATB3
#define MOTOR_MODE1        LATDbits.LATD5
#define MOTOR_MODE2        LATDbits.LATD6
#define MOTOR_CWCCW        LATDbits.LATD7
#define MOTOR_TORQUE1      LATBbits.LATB4
#define MOTOR_TORQUE2      LATBbits.LATB5

#define MOTOR_CLK_TRIS     TRISBbits.RB1   
#define MOTOR_RESET_TRIS   TRISBbits.RB2   
#define MOTOR_ENABLE_TRIS  TRISBbits.RB3
#define MOTOR_MODE1_TRIS   TRISDbits.RD5
#define MOTOR_MODE2_TRIS   TRISDbits.RD6
#define MOTOR_CWCCW_TRIS   TRISDbits.RD7
#define MOTOR_TORQUE1_TRIS TRISBbits.RB4
#define MOTOR_TORQUE2_TRIS TRISBbits.RB5

//---- Rkdc ---------------------------------------------------------------
#define LED1      LATDbits.LATD2    //
#define LED2      LATDbits.LATD1    //
#define LED3      LATDbits.LATD0    //

#define LED1_TRIS TRISDbits.RD2
#define LED2_TRIS TRISDbits.RD1
#define LED3_TRIS TRISDbits.RD0

#define LED_ON  1
#define LED_OFF 0

//-----@[^[GR[_@ ------------------------------------------------
#define RENCODER_A   PORTEbits.RE0
#define RENCODER_B   PORTEbits.RE1

#define RENCODER_A_TRIS  TRISEbits.RE0
#define RENCODER_B_TRIS  TRISEbits.RE1

enum type_encoder_direction { CW, CCW, ENCODER_STOP };
enum type_encoder_direction rotary_encoder(void);
void rotary_encoder_init();

//----- XCb` --------------------------------------------------------
#define SW_OB1           PORTCbits.RC0
#define SW_OB2           PORTCbits.RC1
#define SW_OB3           PORTBbits.RB0
#define SLIDE_SW         PORTEbits.RE2
#define SW_EXT1          PORTCbits.RC2
#define SW_EXT2          PORTDbits.RD3
#define SW_EXT3          PORTDbits.RD4
#define SW_EXT4          PORTCbits.RC6
#define SW_EXT5          PORTCbits.RC7

#define SW_OB1_TRIS      TRISCbits.RC0
#define SW_OB2_TRIS      TRISCbits.RC1
#define SW_OB3_TRIS      TRISBbits.RB0
#define SLIDE_SW_TRIS    TRISEbits.RE2
#define SW_EXT1_TRIS     TRISCbits.RC2
#define SW_EXT2_TRIS     TRISDbits.RD3
#define SW_EXT3_TRIS     TRISDbits.RD4
#define SW_EXT4_TRIS     TRISCbits.RC6
#define SW_EXT5_TRIS     TRISCbits.RC7

#define No_SW_OB1        1
#define No_SW_ESC        1
#define No_SW_OB2        2
#define No_SW_CURSOR     2
#define No_SW_OB3        3
#define No_SW_ENTER      3
#define No_SW_EXT1       4
#define No_SW_EXT2       5
#define No_SW_EXT3       6
#define No_SW_EXT4       7
#define No_SW_EXT5       8
#define No_SW_SOFT1      9
#define No_SW_SOFT2     10
#define No_SW_SOFT3     11
#define No_SW_SOFT4     12
#define No_SW_SOFT5     13
#define No_SW_SOFT6     14
#define No_SW_SOFT7     15

#define SW_ON      0
#define SW_OFF     1

#define SW_ENABLE  1
#define SW_DISABLE 0

#define NUMBER_OF_SW 15

// unsigned char Sw_cursor_status;  // SW_ENABLE,SW_DISABLE
// unsigned char Sw_esc_status;     // SW_ENABLE,SW_DISABLE
// unsigned char Sw_enter_status;   // SW_ENABLE,SW_DISABLE
boolean  Switch_enable;
short    Switch_disable_timer; // `^Oh~^C}[

#define SW_DISABLE_TIME 200  // 1=1msec

char hw_switch_test( void );
// unsigned char Sw_timer;          // `^Oh~^C}[
  
//-------- ^C}[ ---------------------------------------------------------------
#define timer_start  1
#define timer_stop   0

//-------- TMR0 --------------------------------------------------
#define Timer0_overflowd              INTCONbits.TMR0IF
#define Timer0_interrupt_enable       INTCONbits.TMR0IE
#define Timer0_priority               INTCON2bits.TMR0IP
#define Timer0_enable                 T0CONbits.TMR0ON
#define Timer0_mode_8bit              T0CONbits.T08BIT
#define Timer0_use_external_clock     T0CONbits.T0CS
#define Timer0_prescale_NOT_assign    T0CONbits.PSA
#define Timer0_prescale_bit2          T0CONbits.T0PS2
#define Timer0_prescale_bit1          T0CONbits.T0PS1
#define Timer0_prescale_bit0          T0CONbits.T0PS0

//-------- TMR1 --------------------------------------------------
#define Timer1_overflowd              PIR1bits.TMR1IF 
#define Timer1_interrupt_enable       PIE1bits.TMR1IE 
#define Timer1_prescale_bit0          T1CONbits.T1CKPS0
#define Timer1_prescale_bit1          T1CONbits.T1CKPS1
#define Timer1_use_external_clock     T1CONbits.TMR1CS
#define Timer1_enable                 T1CONbits.TMR1ON
#define Timer1_priority               IPR1bits.TMR1IP

//-------- TMR2 -----------------------------------------------
#define Timer2_compare_match          PIR1bits.TMR2IF 
#define Timer2_interrupt_enable       PIE1bits.TMR2IE
#define Timer2_priority               IPR1bits.TMR2IP
#define Timer2_postscale_bit3         T2CONbits.T2OUTPS3
#define Timer2_postscale_bit2         T2CONbits.T2OUTPS2
#define Timer2_postscale_bit1         T2CONbits.T2OUTPS1
#define Timer2_postscale_bit0         T2CONbits.T2OUTPS0
#define Timer2_enable                 T2CONbits.TMR2ON
#define Timer2_prescale_bit1          T2CONbits.T2CKPS1
#define Timer2_prescale_bit0          T2CONbits.T2CKPS0
#define Timer2_prescale               T2CONbits.T2CKPS

//-------- TMR3iPԊԊu) ----------------------------------------------
#define Timer3_overflowd              PIR2bits.TMR3IF
#define Timer3_interrupt_enable       PIE2bits.TMR3IE
#define Timer3_priority               IPR2bits.TMR3IP
#define Timer3_use_external_clock     T3CONbits.TMR3CS
#define Timer3_enable                 T3CONbits.TMR3ON
#define Timer3_mode_16bit             T3CONbits.RD16
#define Timer3_not_sync_ext_clock     T3CONbits.T3SYNC
#define Timer3_prescale_bit1          T3CONbits.T3CKPS1
#define Timer3_prescale_bit0          T3CONbits.T3CKPS0

//-------- CCP Module Clock Source -------------------------------------------
#define CCP_clock_source_bit2         T3CONbits.T3CCP2
#define CCP_clock_source_bit1         T3CONbits.T3CCP1

//-------- Capture/Compare/PWM Module ----------------------------------------
#define CCP1_mode_bit3                CCP1CONbits.CCP1M3
#define CCP1_mode_bit2                CCP1CONbits.CCP1M2
#define CCP1_mode_bit1                CCP1CONbits.CCP1M1
#define CCP1_mode_bit0                CCP1CONbits.CCP1M0
#define CCP1_compare_match_occurred   PIR1bits.CCP1IF
#define CCP1_interrupt_enable         PIE1bits.CCP1IE
#define CCP1_priority                 IPR1bits.CCP1IP

//-------- PORTB -------------------------------------------------------------
#define PortB_pullups_disable         INTCON2bits.NOT_RBPU

//----- 荞 -----------------------------------------------------------------------
#define High_priority 1
#define Low_priority  0

void InterruptVectorLow (void);
void InterruptVectorHigh (void);
void Low_priority_interrupt (void);
void High_priority_interrupt (void);

//----- Zp֐ -------------------------------------------------------------------
unsigned long pow_long( unsigned long, unsigned char );

//-----@^ϊp̋p ------------------------------------------------------------
typedef union {
  unsigned long  Lval;
  unsigned short Wval[2];
  unsigned char  Bval[4];
} long_union; 

typedef union {
	short          Sval;
	unsigned int   Ival;
  unsigned short Wval;
  unsigned char  Bval[2];
} short_union; 

//---- Ƀ[^[p̕ϐȂ ---------------------------------------
void motor_driver_disable(void);
void motor_driver_enable(void);
void display_repeat_run( unsigned short );

enum type_setup             { DISPLAY, UPDATE };
enum type_motor_direction   { M_CW, M_CCW, M_HALT };   // M_CWFv@M_CCW:v M_HALT:Ȃ
enum type_time_unit         { USEC, MSEC, SEC };
enum type_driver_mode       { x1, x2, x8, x16 };

typedef struct {
  enum type_motor_direction  cwccw;     // 1byte@@
  unsigned long              interval;  // 4byte
  enum type_time_unit        unit;      // 1byte @usec:ʕb msec:~b sec:b
  unsigned long              step;      // 4byte  -1: 0:]I
  unsigned char              priority;  // 1byte
  unsigned char              torque;    // 1byte  100,75,50,25%
  unsigned short             repeat;    // 2byte  JԂ
  unsigned char              next;      // 1byte
  unsigned char              dummy;
} type_action;                          // total 16 bytes

type_action Action;
boolean     Off_mode;    // ~̓dJbg

#define NUMBER_OF_PARAMETER 8

#define POWER_OFF_TIME     500    // 1=1msec

// unsigned short TMR0_over_flow_count;
unsigned char  Current_priority;
unsigned long  CCP1_match_count;
unsigned long  Step_interval_counter;
unsigned long  Step_counter;
unsigned short Repeat_count;
unsigned short Repeat_sw_no;
boolean        Motor_run;      // HALTi҂ԁj̎true
short          Motor_power_off_timer;
boolean        Motor_power_off_wait; 

//----- ddoqnll ----------------------------------------------------
#pragma romdata eedata_scn1=0xf00000 
	rom char eedata_values[2] = {x16,true};
#pragma romdata

#pragma romdata eedata_scn2=0xf00010 
 	rom char eedata_sw_ob1[16] = {M_CCW,44,1,0,0,USEC,128,12,0,0,1,100,0,0,0,0xFF};   // No:1
	rom char eedata_sw_ob2[16] = {M_CCW,88,2,0,0,USEC,128,12,0,0,1,100,0,0,0,0xFF};   // No:2 
	rom char eedata_sw_ob3[16] = {M_CW, 44,1,0,0,USEC,128,12,0,0,1,100,0,0,0,0xFF};   // No:3 
	rom char eedata_sw_ex1[16] = {M_CCW,44,1,0,0,USEC,128,12,0,0,1,100,0,0,0,0xFF};   // No:4 
	rom char eedata_sw_ex2[16] = {M_CCW,44,1,0,0,USEC,128,12,0,0,1,100,0,0,0,0xFF};   // No:5 
	rom char eedata_sw_ex3[16] = {M_CCW,44,1,0,0,USEC,128,12,0,0,1,100,0,0,0,0xFF};   // No:6 
	rom char eedata_sw_ex4[16] = {M_CCW,44,1,0,0,USEC,128,12,0,0,1,100,0,0,0,0xFF};   // No:7 
	rom char eedata_sw_ex5[16] = {M_CCW,44,1,0,0,USEC,128,12,0,0,1,100,0,0,0,0xFF};   // No:8 
	rom char eedata_sw_sf1[16] = {M_CCW,44,1,0,0,USEC,128,12,0,0,1,100,0,0,0,0xFF};   // No:9 
	rom char eedata_sw_sf2[16] = {M_CCW,44,1,0,0,USEC,128,12,0,0,1,100,0,0,0,0xFF};   // No:10 
	rom char eedata_sw_sf3[16] = {M_CCW,44,1,0,0,USEC,128,12,0,0,1,100,0,0,0,0xFF};   // No:11 
	rom char eedata_sw_sf4[16] = {M_CCW,44,1,0,0,USEC,128,12,0,0,1,100,0,0,0,0xFF};   // No:12 
	rom char eedata_sw_sf5[16] = {M_CCW,44,1,0,0,USEC,128,12,0,0,1,100,0,0,0,0xFF};   // No:13 
	rom char eedata_sw_sf6[16] = {M_CCW,44,1,0,0,USEC,128,12,0,0,1,100,0,0,0,0xFF};   // No:14 
	rom char eedata_sw_sf7[16] = {M_CCW,44,1,0,0,USEC,128,12,0,0,1,100,0,0,0,0xFF};   // No:15 
#pragma romdata

#define ADDR_DRIVER_MODE 0
#define ADDR_OFF_MODE    1   

//----------------------------------------------------------------------------
// 荞݃xN^[ݒ
//----------------------------------------------------------------------------
#ifdef USE_BOOTLOADER
#pragma code InterruptVectorLow = 0x2018
#else
#pragma code InterruptVectorLow = 0x0018
#endif
void InterruptVectorLow (void)
{
  _asm
    goto Low_priority_interrupt   //jump to interrupt routine
  _endasm
}

#ifdef USE_BOOTLOADER
#pragma code InterruptVectorHigh = 0x2008
#else
#pragma code InterruptVectorHigh = 0x0008
#endif
void InterruptVectorHigh (void)
{
  _asm
    goto High_priority_interrupt  //jump to interrupt routine
  _endasm
}

#pragma code
#pragma interruptlow Low_priority_interrupt
#pragma interrupt    High_priority_interrupt


//----------------------------------------------------------------------------
// unsigned long ׂ
//----------------------------------------------------------------------------
unsigned long pow_long( unsigned long val, unsigned char pv )
{
  unsigned char i;
  unsigned long return_val;

  if( pv == 0 ) {
    return 1;
  }

  return_val = 1;
  for( i = 0 ; i < pv ; i++ ) {
    return_val *= val;
  }

  return return_val;
}


//-------------------------------------------------------------------------------
// delay  usec     Clock = 48Mhz    ̊֐͌덷
//-------------------------------------------------------------------------------
void delay_us(unsigned short delay)
{
  unsigned short i;

  for( i = 0 ; i <= delay ; i++ ) {
    Delay1TCY();
    Delay1TCY();
    Delay1TCY();
    Delay1TCY();
    Delay1TCY();
    Delay1TCY();
    Delay1TCY();
    Delay1TCY();
    Delay1TCY();
    Delay1TCY();
  }
}


//-------------------------------------------------------------------------------
// delay  msec           Clock = 48Mhz
//-------------------------------------------------------------------------------
void delay_ms(unsigned short delay)
{
  unsigned short i;

  for( i = 0 ; i <= delay ; i++ ) {
    Delay1KTCYx(12);
  }
}


//-------------------------------------------------------------------------------
// tfBXvCɂSrbgM@@v`hs
//-------------------------------------------------------------------------------
void lcd_send_nibble(unsigned char nibble, unsigned short nsec )
{
  unsigned char i;
  unsigned char bit_val;

  if ( (nibble & 0x01) == 0x01 ) { LCD_DB4 = 1; } else { LCD_DB4 = 0; } nibble >>= 1;
  if ( (nibble & 0x01) == 0x01 ) { LCD_DB5 = 1; } else { LCD_DB5 = 0; } nibble >>= 1;
  if ( (nibble & 0x01) == 0x01 ) { LCD_DB6 = 1; } else { LCD_DB6 = 0; } nibble >>= 1;
  if ( (nibble & 0x01) == 0x01 ) { LCD_DB7 = 1; } else { LCD_DB7 = 0; }

  LCD_E = 1;
    Delay1TCY();
    Delay1TCY();
    Delay1TCY();
    Delay1TCY();
    Delay1TCY();
  LCD_E = 0;

  while( nsec > 0 ) {
    delay_us(1); 
    nsec--;
  }
} 

//-------------------------------------------------------------------------------
// tfBXvCɂPoCgM
//-------------------------------------------------------------------------------
void lcd_byte(unsigned char rs,unsigned char ch )
{
  unsigned char ch_bit;
  unsigned char ch_tmp;
  unsigned char i;
  
  ch_tmp = ch;
  
  if ( rs == 1 ) {
    LCD_RS = 1;
  } else {
    LCD_RS = 0;
  }

  for( i = 0; i <= 1 ; i++ ) {
    if ( (ch_tmp & 0b10000000) == 0x0 ) { LCD_DB7 = 0; } else { LCD_DB7 = 1; } ch_tmp <<= 1;
    if ( (ch_tmp & 0b10000000) == 0x0 ) { LCD_DB6 = 0; } else { LCD_DB6 = 1; } ch_tmp <<= 1;
    if ( (ch_tmp & 0b10000000) == 0x0 ) { LCD_DB5 = 0; } else { LCD_DB5 = 1; } ch_tmp <<= 1;
    if ( (ch_tmp & 0b10000000) == 0x0 ) { LCD_DB4 = 0; } else { LCD_DB4 = 1; } ch_tmp <<= 1;
    LCD_E = 1;
      Delay1TCY();
      Delay1TCY();
      Delay1TCY();
    LCD_E = 0;
  }  
  
  if (ch <= 0b00000011) {
    delay_us(1640);
  } else {
    delay_us(  46);
  }

  LCD_RS = 0;
}

//-------------------------------------------------------------------------------
// tfBXvCN
//-------------------------------------------------------------------------------
void lcd_clear()
{
  lcd_send_nibble( 0b0000,     2 );
  lcd_send_nibble( 0b0001,  1640 );
}

//-------------------------------------------------------------------------------
// tfBXvCN@i
//-------------------------------------------------------------------------------
void lcd_clear_top(void)
{
  lcd_locate(0,0);
  lcd_const_string("        ");
  lcd_locate(0,0);
}

//-------------------------------------------------------------------------------
// tfBXvCN@i
//-------------------------------------------------------------------------------
void lcd_clear_bottom(void)
{
  lcd_locate(0,1);
  lcd_const_string("        ");
  lcd_locate(0,1);
}

//-------------------------------------------------------------------------------
// tfBXvC̃J[\z[|WV
//-------------------------------------------------------------------------------
void lcd_home(void)
{
  lcd_byte( 0, 0b00000010 );
}

//-------------------------------------------------------------------------------
// tfBXvC̃J[\
//-------------------------------------------------------------------------------
void lcd_cursor( enum type_LCD_cursor cursor,  enum type_LCD_cursor blink )
{
  unsigned char ch;

  ch = 0b1100;
  if ( cursor == CURSOR_ON ) {
    ch = ch | 0b0010;
  }
  if ( blink == BLINK_ON ) {
    ch = ch | 0b0001;
  }

  lcd_send_nibble( 0b0000,   2 );
  lcd_send_nibble( ch    ,  46 );
}

//-------------------------------------------------------------------------------
// tfBXvC
//-------------------------------------------------------------------------------
void lcd_init()
{
  delay_ms(50);
  LCD_E_TRIS   = PORT_OUTPUT;
  LCD_RS_TRIS  = PORT_OUTPUT;
  LCD_DB7_TRIS = PORT_OUTPUT;
  LCD_DB6_TRIS = PORT_OUTPUT;
  LCD_DB5_TRIS = PORT_OUTPUT;
  LCD_DB4_TRIS = PORT_OUTPUT;

  LCD_E   = 0;
  LCD_RS  = 0;

  delay_ms(30);

  lcd_send_nibble( 0b0011, 4100 );
  lcd_send_nibble( 0b0011,  100 );

  lcd_send_nibble( 0b0010,   40 );  // function set
  lcd_send_nibble( 0b0010,   40 );
  lcd_send_nibble( 0b1000,   40 );

  lcd_cursor( CURSOR_OFF , BLINK_OFF );
  lcd_clear();
}

//-------------------------------------------------------------------------------
// tfBXvCɂP\
//-------------------------------------------------------------------------------
void lcd_char( unsigned char ch )
{
  lcd_byte( 1, ch );
}

//-------------------------------------------------------------------------------
// tfBXvCɕ\
//-------------------------------------------------------------------------------
void lcd_string( char *buf )
{
  while( *buf != 0x0 ) {
    lcd_char( *buf );
    buf++;
  }
}

//-------------------------------------------------------------------------------
// tfBXvCɒ萔\
//-------------------------------------------------------------------------------
void lcd_const_string( const rom char *buf )
{
  while( *buf != 0x0 ) {
    lcd_char( *buf );
    buf++;
  }
}

//-------------------------------------------------------------------------------
// tfBXvC̃J[\ʒuύX
//
// x -> cusor x posision     range 0 to 7
// y -> cusor y posision     range 0 to 1
//
// ex
//   lcd_locate( 0, 0 )  <- home posision (upper left)
//-------------------------------------------------------------------------------
void lcd_locate( unsigned char x, unsigned char y )
{
  unsigned char addr;

  addr = 0b10000000;
  if ( y == 1 ) {
    addr += 0x40;
  }
  addr += x;

  lcd_byte( 0, addr );
}


//-------------------------------------------------------------------------------
// QoCgϐPUiłkbcɕ\ (oOp֐debug routine)
//-------------------------------------------------------------------------------
void display_short( unsigned short arg, unsigned char row )
{
  char buf[5];

  lcd_locate(0,row);  
  sprintf( buf, (const far rom char *)"%04x", arg );
  lcd_string(buf);
}

void motor_driver_disable2(void)
{
}

//****************************************************************************
// Dx 荞݃[`
//****************************************************************************
void High_priority_interrupt ()
{
  //---- CCP1_compare_match ----------
  if( CCP1_compare_match_occurred == true ) {
    CCP1_compare_match_occurred = false;
    Step_interval_counter--;
    if( Step_interval_counter == 0 ) {
	    
	    Step_interval_counter = CCP1_match_count;
	    if( Step_counter > 0 ) {
        Step_counter--;
   	    MOTOR_CLOCK = 1;
  	    TMR2 = 0;
        Timer2_enable = true;
	      Timer2_interrupt_enable = true;
      } else {
        Motor_run = false;
        Timer1_enable = false;
        CCP1_interrupt_enable = false;
        if( Off_mode == true ) {
	        Motor_power_off_timer = POWER_OFF_TIME; 
	        Motor_power_off_wait = true;
        }
      }
    }
  }
}

//****************************************************************************
// Dx@荞݃[`
//****************************************************************************
void Low_priority_interrupt ()
{

	//---- TMR2 compare match CCP1̊荞݂CLOCKPɂȂĂUOʕbɂɗ ----
  if ( Timer2_compare_match == true ) {
    Timer2_compare_match = false;
    Timer2_enable = false;
    Timer2_interrupt_enable = false;
    MOTOR_CLOCK = 0;
  }
	  
  //---- TMR3 overflow (1~bɂslqR͂ӂ) ----------
  if ( Timer3_overflowd == true ) {
    Timer3_overflowd = false;
    TMR3H = 0xD1;
    TMR3L = 0x20;

    //--------- `^Oh~ ---------------------------------
    // XCb`OFFɂȂĂ SW_DISABLE_TIME o߂ȂƃXCb`͗LɂȂȂ
    if( Switch_enable == false ) {
	    if( SW_OB1  == SW_ON ) { Switch_disable_timer = SW_DISABLE_TIME; }
	    if( SW_OB2  == SW_ON ) { Switch_disable_timer = SW_DISABLE_TIME; }
	    if( SW_OB3  == SW_ON ) { Switch_disable_timer = SW_DISABLE_TIME; }
	    if( SW_EXT1 == SW_ON ) { Switch_disable_timer = SW_DISABLE_TIME; }
	    if( SW_EXT2 == SW_ON ) { Switch_disable_timer = SW_DISABLE_TIME; }
	    if( SW_EXT3 == SW_ON ) { Switch_disable_timer = SW_DISABLE_TIME; }
	    if( SW_EXT4 == SW_ON ) { Switch_disable_timer = SW_DISABLE_TIME; }
	    if( SW_EXT5 == SW_ON ) { Switch_disable_timer = SW_DISABLE_TIME; }
	    
	    if( Switch_disable_timer > 0 ) {
	      Switch_disable_timer--;
	    }
  	  if( Switch_disable_timer == 0 ) {
        Switch_enable = true;
      }
	  }
	  
	  if( Motor_power_off_timer > 0 ) {
		  Motor_power_off_timer--;
		}
  }
}


//----------------------------------------------------------------------------
// [^[GR[_Tu[`
//----------------------------------------------------------------------------
struct {
  unsigned a_port:1;
  unsigned b_port:1;
  unsigned a_lock:1;
  unsigned b_lock:1;
  unsigned a_tmp:1;
  unsigned b_tmp:1;
} Re_stat;

void rotary_encoder_init()
{
  RENCODER_A_TRIS = PORT_INPUT;
  RENCODER_B_TRIS = PORT_INPUT;

  Re_stat.a_port = RENCODER_A;
  Re_stat.b_port = RENCODER_B;

  Re_stat.a_lock = false;
  Re_stat.b_lock = false;
}

enum type_encoder_direction rotary_encoder()
{
  enum type_encoder_direction rotary_stat;  

  Re_stat.a_tmp = RENCODER_A;
  Re_stat.b_tmp = RENCODER_B;
 
  rotary_stat  = ENCODER_STOP;

  //--- CW ----------------
  if ((Re_stat.a_port == 1) && (Re_stat.a_tmp == 0) && (Re_stat.b_tmp == 1)) {
    Re_stat.a_lock = true;
    Re_stat.b_lock = false;
  }
  if (Re_stat.a_lock == true ) {
    if ((Re_stat.a_tmp == 0) && (Re_stat.b_tmp == 0) ) {
      Re_stat.a_lock = false;
      rotary_stat = CW;
    }
  }

  //--- CCW ----------------
  if ((Re_stat.b_port == 1) && (Re_stat.b_tmp == 0) && (Re_stat.a_tmp == 1)) {
    Re_stat.b_lock = true;
    Re_stat.a_lock = false;
  }
  if (Re_stat.b_lock == true ) {
    if ((Re_stat.a_tmp == 0) && (Re_stat.b_tmp == 0) ) {
      Re_stat.b_lock = false;
      rotary_stat = CCW;
    }
  }
  
  Re_stat.a_port = Re_stat.a_tmp;
  Re_stat.b_port = Re_stat.b_tmp;


  return rotary_stat;       // CW / CCW / ENCODER_STOP
}

//----------------------------------------------------------------------------
// ȂQoCg@t\
//----------------------------------------------------------------------------
void display_ushort( unsigned short val, unsigned char length, unsigned char col, unsigned char row ) 
{
  char buf[9];
  lcd_locate(col,row);
  switch( length ) {
    case 1: sprintf( buf, (rom char far const *)"%1d", val ); break;
    case 2: sprintf( buf, (rom char far const *)"%2d", val ); break;
    case 3: sprintf( buf, (rom char far const *)"%3d", val ); break;
    case 4: sprintf( buf, (rom char far const *)"%4d", val ); break;
    case 5: sprintf( buf, (rom char far const *)"%5d", val ); break;
 }
 lcd_string( buf );
}


//----------------------------------------------------------------------------
// [^[GR[_ɂ鐔l
//
// init_val:l 
// min_val:͉\ȍŏl
// max_val:͉\ȍől
//----------------------------------------------------------------------------
unsigned long scan_digit(
 unsigned long init_val,
 unsigned long min_val,
 unsigned long max_val)
{
  unsigned char turn;     // CW,CCW@[^[GR[_]
  unsigned char cursor;   // J[\ʒu
  unsigned long value;    // ͂li[ϐ
  unsigned char digits;   // 
  unsigned long tmp_long; // e|ϐ
  char buf[9];            // t\pobt@

  lcd_locate(7,1);
  lcd_cursor(CURSOR_ON,BLINK_OFF);

  tmp_long = max_val;
  digits = 0;
  while( tmp_long > 0 ) {  // őľvZ
    tmp_long /= 10;
    digits++;
  }

  cursor = 0;  
  value = init_val;
  while( No_SW_ENTER != hw_switch_test()) {
    turn = rotary_encoder();
    if( turn != ENCODER_STOP ) {
      if( turn == CW ) {
        if( value + pow_long(10,cursor) > max_val ) {
          value = max_val;
        } else {
          value += pow_long(10,cursor);
        }
      } else {
        if( value < ( pow_long(10,cursor) + min_val ) ) {
          value = min_val;
        } else {
          value -= pow_long(10,cursor);
        }
      }

      sprintf( buf,(rom char far const *)"%8ld", value  );
      lcd_locate(0,1);
      lcd_string( buf );
      lcd_locate(7-cursor,1);
    }

    if( No_SW_CURSOR == hw_switch_test()) {
   	  Switch_enable = false;
      if( cursor == (digits - 1) ) {
        cursor = 0;
      } else {
        cursor++;
      }
      lcd_locate(7-cursor,1);
    }
  }
  Switch_enable = false;
  lcd_cursor(CURSOR_OFF,BLINK_OFF);

  return value;
}

//----------------------------------------------------------------------------
// QoCgddoqnlǂݏo
//----------------------------------------------------------------------------
unsigned int Read_W_eep( unsigned int addr )
{
	short_union udata;
	
	udata.Bval[0] = Read_b_eep( addr + 0 );
	udata.Bval[1] = Read_b_eep( addr + 1 );
	
	return udata.Ival;
}

//----------------------------------------------------------------------------
// QoCgddoqnl֏
//----------------------------------------------------------------------------
void Write_W_eep( unsigned int addr, unsigned int value )
{
	short_union udata;
	
	udata.Ival = value;
	Write_b_eep( addr+0, udata.Bval[0] );
	Write_b_eep( addr+1, udata.Bval[1] );
}

//----------------------------------------------------------------------------
// SoCgddoqnlǂݏo
//----------------------------------------------------------------------------
unsigned long Read_L_eep( unsigned int addr )
{
	long_union udata;
	
	udata.Bval[0] = Read_b_eep( addr + 0 );
	udata.Bval[1] = Read_b_eep( addr + 1 );
	udata.Bval[2] = Read_b_eep( addr + 2 );
	udata.Bval[3] = Read_b_eep( addr + 3 );
			
	return udata.Lval;
}

//----------------------------------------------------------------------------
// SoCgddoqnl֏
//----------------------------------------------------------------------------
void Write_L_eep( unsigned int addr, unsigned long value )
{
	long_union udata;
	
	udata.Lval = value;
	Write_b_eep( addr + 0, udata.Bval[0] );
	Write_b_eep( addr + 1, udata.Bval[1] );
	Write_b_eep( addr + 2, udata.Bval[2] );
	Write_b_eep( addr + 3, udata.Bval[3] );
}

//----------------------------------------------------------------------------
// EEPROMXCb`Ƃ̓̓ǂݏo
//----------------------------------------------------------------------------
void read_eep_action( short switch_no )
{
	unsigned char eepaddr;
	
	eepaddr = switch_no * sizeof(type_action);
  Action.cwccw        = Read_b_eep( eepaddr + offsetof( type_action, cwccw    ));
  Action.interval     = Read_L_eep( eepaddr + offsetof( type_action, interval ));
  Action.unit         = Read_L_eep( eepaddr + offsetof( type_action, unit     ));
  Action.step         = Read_L_eep( eepaddr + offsetof( type_action, step     ));
  Action.priority     = Read_b_eep( eepaddr + offsetof( type_action, priority ));
  Action.torque       = Read_b_eep( eepaddr + offsetof( type_action, torque   ));
  Action.repeat       = Read_W_eep( eepaddr + offsetof( type_action, repeat   ));
  Action.next         = Read_b_eep( eepaddr + offsetof( type_action, next     ));
}	


//----------------------------------------------------------------------------
// EEPROMփXCb`Ƃ̓
//----------------------------------------------------------------------------
void write_eep_action( short switch_no )
{
	unsigned char eepaddr;
	
	eepaddr = switch_no * sizeof(type_action);
  Write_b_eep( eepaddr + offsetof( type_action, cwccw    ), Action.cwccw    );
  Write_L_eep( eepaddr + offsetof( type_action, interval ), Action.interval );
  Write_L_eep( eepaddr + offsetof( type_action, unit     ), Action.unit     );
  Write_L_eep( eepaddr + offsetof( type_action, step     ), Action.step     );
  Write_b_eep( eepaddr + offsetof( type_action, priority ), Action.priority );
  Write_b_eep( eepaddr + offsetof( type_action, torque   ), Action.torque   );
  Write_W_eep( eepaddr + offsetof( type_action, repeat   ), Action.repeat   );
  Write_b_eep( eepaddr + offsetof( type_action, next     ), Action.next     );
}	


//----------------------------------------------------------------------------
// Xebvi]pxj̐ݒ
//----------------------------------------------------------------------------
void display_step_num( unsigned long step )
{
  char buf[9];

  sprintf( buf, (rom char far const *)"%8ld", step );
  lcd_locate(0,1);
  lcd_string( buf );
}

void display_step( unsigned long step )
{
  lcd_home();
  lcd_const_string("STEP");
  display_step_num( step );
}

unsigned long get_step(unsigned long  step )
{
  unsigned long tmp_step;
  unsigned char turn;     // CW,CCW
  
  tmp_step = scan_digit( step, 1, 99999999 );

  return tmp_step;
}


//----------------------------------------------------------------------------
// XCb`Ƃ̓̊Tv\
//----------------------------------------------------------------------------
void display_switch_no( unsigned char switch_no )
{
  switch( switch_no ) {
    case No_SW_OB1:   lcd_const_string("ON1"); break;
    case No_SW_OB2:   lcd_const_string("ON2"); break;
    case No_SW_OB3:   lcd_const_string("ON3"); break;
    case No_SW_EXT1:  lcd_const_string("EX1"); break;
    case No_SW_EXT2:  lcd_const_string("EX2"); break;
    case No_SW_EXT3:  lcd_const_string("EX3"); break;
    case No_SW_EXT4:  lcd_const_string("EX4"); break;
    case No_SW_EXT5:  lcd_const_string("EX5"); break;
    case No_SW_SOFT1: lcd_const_string("SF1"); break;
    case No_SW_SOFT2: lcd_const_string("SF2"); break;
    case No_SW_SOFT3: lcd_const_string("SF3"); break;
    case No_SW_SOFT4: lcd_const_string("SF4"); break;
    case No_SW_SOFT5: lcd_const_string("SF5"); break;
    case No_SW_SOFT6: lcd_const_string("SF6"); break;
    case No_SW_SOFT7: lcd_const_string("SF7"); break;
  }
}	
/*
void display_step(void)
{
  char buf[9];	// t\pobt@

  lcd_locate(0,1);	
  sprintf( buf,(rom char far const *)"%8d  ", Action.step );
}
*/

// EX
// ON1
//     1234


void display_switch_action( unsigned char switch_no )
{
//  char buf[5];	// t\pobt@

  lcd_clear();
  display_switch_no( switch_no );
  read_eep_action( switch_no );
//  display_repeat_run(Action.repeat); 
lcd_locate(5,0); 
display_switch_no( Action.next );
/*
	switch( Action.cwccw ) {
    case M_CW:   lcd_const_string("  CW");  break;
    case M_CCW:  lcd_const_string(" CCW");  break;
    case M_HALT: lcd_const_string("HALT");  break;
  }
*/  
//  lcd_string( buf );
  display_step_num(Action.step);
}


//----------------------------------------------------------------------------
// ]̐ݒ
//----------------------------------------------------------------------------
void display_direction( enum type_motor_direction direction )
{
	lcd_home();
  lcd_const_string("DIRC");
  lcd_clear_bottom();
  switch( direction ) {
    case M_CW:   lcd_const_string("CW  ");  break;
    case M_CCW:  lcd_const_string("CCW ");  break;
    case M_HALT: lcd_const_string("HALT");  break;
  }
}

enum type_motor_direction get_direction(enum type_motor_direction direction)
{
  enum type_motor_direction tmp_direction;
  unsigned char turn;     // CW,CCW
  
  tmp_direction = direction;
  while( true ) {
    turn = rotary_encoder();
    if( turn != ENCODER_STOP ) {  
      if( turn == CW ) {
        switch( tmp_direction ) {
          case M_CW:   tmp_direction = M_CCW;  break;
          case M_CCW:  tmp_direction = M_HALT; break;
        }
      } else {  // CCW
        switch( tmp_direction ) {
          case M_HALT: tmp_direction = M_CCW;  break;
          case M_CCW:  tmp_direction = M_CW;   break;
        }
      }
      display_direction( tmp_direction ); 
    }
    if( No_SW_ESC == hw_switch_test() ) {
	    Switch_enable = false;
      display_direction( direction );
 	    return direction;
	  }
	  if( No_SW_ENTER == hw_switch_test() ) {
		  Switch_enable = false;
		  return tmp_direction;
		}  
  } 
}

//----------------------------------------------------------------------------
// XebvԊuԂ̒Pʂ̐ݒ
//----------------------------------------------------------------------------
void display_unit( enum type_time_unit unit )
{
  lcd_home();
  lcd_const_string("UNIT");	
  lcd_clear_bottom();
  switch( unit ) {
    case USEC: lcd_const_string("MicroSec"); break;
    case MSEC: lcd_const_string("MilliSec"); break;
    case SEC:  lcd_const_string("Second  "); break;
  }
}

enum type_time_unit get_unit(enum type_time_unit unit)
{
  enum type_time_unit tmp_unit;
  unsigned char turn;     // CW,CCW
  
  tmp_unit = unit;
  while( true ) {
    turn = rotary_encoder();
    if( turn != ENCODER_STOP ) {  
      if( turn == CW ) {
	      switch( tmp_unit ) {
		      case SEC:  tmp_unit = MSEC; break;
          case MSEC: tmp_unit = USEC; break;
        }   
      } else {  // CCW
        switch( tmp_unit ) {
          case USEC: tmp_unit = MSEC; break;
          case MSEC: tmp_unit = SEC;  break;
        }  
      }
      display_unit( tmp_unit );
    }
    if( No_SW_ESC == hw_switch_test() ) {
	    Switch_enable = false;
      display_unit( unit );
 	    return unit;
	  }
	  if( No_SW_ENTER == hw_switch_test() ) {
		  Switch_enable = false;
		  return tmp_unit;
		}  
  } 
}


//----------------------------------------------------------------------------
// XebvԊuԂ̐ݒ
//----------------------------------------------------------------------------
void display_interval( unsigned long interval )
{
  char buf[9];
  
  lcd_home();
  lcd_const_string("ITVL");
  sprintf( buf, (rom char far const *)"%8ld", interval );
  lcd_locate(0,1);
  lcd_string( buf );
}

unsigned long get_interval(unsigned long  interval, enum type_time_unit unit )
{
  unsigned long tmp_interval;
  unsigned char turn;     // CW,CCW
  
  switch( unit ) {
    case USEC: tmp_interval = scan_digit( interval, 120, 16000  ); break;
    case MSEC: tmp_interval = scan_digit( interval, 1, 99999999 ); break;
    case SEC:  tmp_interval = scan_digit( interval, 1, 99999999 ); break;
  }

  return tmp_interval;
}


//----------------------------------------------------------------------------
// JԂ񐔂̐ݒ
//----------------------------------------------------------------------------
void display_repeat( unsigned short repeat )
{
  char buf[9];

  lcd_home();
  lcd_const_string("REPT");
  lcd_clear_bottom();
  sprintf( buf, (rom char far const *)"%8u", repeat );
  lcd_string( buf );
}

void display_repeat_run( unsigned short repeat )
{
  char buf[6];

  lcd_locate(3,0);
  sprintf( buf, (rom char far const *)"%5u", repeat );
  lcd_string( buf );
}


unsigned short get_repeat( unsigned short repeat )
{
  unsigned short tmp_repeat;
  unsigned char turn;     // CW,CCW
  
  tmp_repeat = scan_digit( repeat, 0, 60000 );

  return tmp_repeat;
}


//----------------------------------------------------------------------------
// D揇ʂ̐ݒ
//----------------------------------------------------------------------------
void display_priority( unsigned char priority )
{
  lcd_home();
  lcd_const_string("PRI ");
  lcd_clear_bottom();
  switch( priority ) {
    case 1: lcd_const_string("LOW");  break;
    case 2: lcd_const_string("2");    break;
    case 3: lcd_const_string("3");    break;
    case 4: lcd_const_string("4");    break;
    case 5: lcd_const_string("HIGH"); break;
  }
}

unsigned char get_priority(unsigned char priority)
{
  unsigned char tmp_priority;
  unsigned char turn;     // CW,CCW

  tmp_priority = priority;
  while( true ) {
    display_priority( tmp_priority );
    turn = rotary_encoder();
    if( turn != ENCODER_STOP ) {  
      if( turn == CW ) {
        if( tmp_priority < 5 ) { tmp_priority++; }
      } else {
        if( tmp_priority > 1 ) { tmp_priority--; }
      }
    }
    if( No_SW_ESC == hw_switch_test() ) {
	    Switch_enable = false;
      display_priority( priority );
 	    return priority;
	  }
	  if( No_SW_ENTER == hw_switch_test() ) {
		  Switch_enable = false;
		  return tmp_priority;
		}  
  } 
}


//----------------------------------------------------------------------------
// gN̐ݒ
//----------------------------------------------------------------------------
void display_torque( unsigned char torque )
{
  lcd_home();
  lcd_const_string("TORQ");	
  lcd_clear_bottom();
  switch( torque ) {
    case  25: lcd_const_string("25%");  break;
    case  50: lcd_const_string("50%");  break;
    case  75: lcd_const_string("75%");  break;
    case 100: lcd_const_string("100%"); break;
  }
}


unsigned char  get_torque(unsigned char torque )
{
  unsigned char tmp_torque;
  unsigned char turn;     // CW,CCW

  tmp_torque = torque;
  while( true ) {
    turn = rotary_encoder();
    if( turn != ENCODER_STOP ) {  
      if( turn == CW ) {
        if( tmp_torque < 100 ) { tmp_torque += 25; }
      } else {  // CCW
        if( tmp_torque > 25  ) { tmp_torque -= 25; }
      }
      display_torque( tmp_torque );
    }
    if( No_SW_ESC == hw_switch_test() ) {
	    Switch_enable = false;
      display_torque( torque );
 	    return torque;
	  }
	  if( No_SW_ENTER == hw_switch_test() ) {
		  Switch_enable = false;
		  return tmp_torque;
		}
  }
}


//----------------------------------------------------------------------------
// ̃ANV̐ݒ
//----------------------------------------------------------------------------
void display_next( unsigned char next_sw )
{
  lcd_home();
  lcd_const_string("NEXT");	
  lcd_clear_bottom();
  if( next_sw == 0 ) {
	  lcd_const_string("nothing");
	} else {
    display_switch_no( next_sw );
  } 
}


unsigned char get_next(unsigned char next_sw )
{
  unsigned char tmp_next_sw;
  unsigned char turn;     // CW,CCW

  tmp_next_sw = next_sw;
  while( true ) {
    turn = rotary_encoder();
    if( turn != ENCODER_STOP ) {  
      if( turn == CW ) {
        if( tmp_next_sw < NUMBER_OF_SW ) { tmp_next_sw++; }
      } else {  // CCW
        if( tmp_next_sw > 0            ) { tmp_next_sw--; }
      }
      display_next( tmp_next_sw );
    }
    if( No_SW_ESC == hw_switch_test() ) {
	    Switch_enable = false;
      display_next( next_sw );
 	    return next_sw;
	  }
	  if( No_SW_ENTER == hw_switch_test() ) {
		  Switch_enable = false;
		  return tmp_next_sw;
		}
  }
}

//----------------------------------------------------------------------------
// ㎥[h̕\@@QAP|QA}CNXebvQvP|QA}CNXebvSvP|Q
//----------------------------------------------------------------------------
void display_driver_mode(enum type_driver_mode mode)
{
  lcd_clear();
  lcd_const_string("DRV MODE");
  lcd_locate(0,1);
  switch( mode ) {
	  case x1:  lcd_const_string("x1");  break;
	  case x2:  lcd_const_string("x2");  break;
	  case x8:  lcd_const_string("x8");  break;
	  case x16: lcd_const_string("x16"); break;
  }   
}

void display_sys_mode(void)
{
	enum type_driver_mode driver_mode;
	boolean off_mode;
	
  driver_mode = Read_b_eep( ADDR_DRIVER_MODE );
  off_mode    = Read_b_eep( ADDR_OFF_MODE    );
  lcd_clear();
  lcd_const_string("SYS MODE");
  lcd_locate(0,1);
  switch( driver_mode ) {
	  case x1:  lcd_const_string("x1");  break;
	  case x2:  lcd_const_string("x2");  break;
	  case x8:  lcd_const_string("x8");  break;
	  case x16: lcd_const_string("x16"); break;
  }   
  lcd_locate(4,1);
  switch( off_mode ) {
	  case true:  lcd_const_string("OFF ");  break;
	  case false: lcd_const_string("CONT");  break;
  }
}

enum type_driver_mode get_driver_mode(void)
{
  enum type_driver_mode mode;	
  enum type_driver_mode mode_save;	
  unsigned char turn;     // CW,CCW

  mode_save = Read_b_eep( ADDR_DRIVER_MODE );
  mode = mode_save;
  while( true ) {
    turn = rotary_encoder();
    if( turn != ENCODER_STOP ) {  
      if( turn == CW ) {
	      switch( mode ) {
		      case x1: mode = x2;  break;
		      case x2: mode = x8;  break;
		      case x8: mode = x16; break;
		    }
      } else {  // CCW
	      switch( mode ) {
		      case x2: mode = x1;  break;
		      case x8: mode = x2;  break;
		      case x16: mode = x8; break;
		    } 
      }
      display_driver_mode( mode );
    }
    if( No_SW_ESC == hw_switch_test() ) {
	    Switch_enable = false;
      return mode_save;
	  }
	  if( No_SW_ENTER == hw_switch_test() ) {
		  Switch_enable = false;
		  return mode;
		}
  }	
}


//----------------------------------------------------------------------------
// [^[]~̓dON/OFFݒ
//----------------------------------------------------------------------------
void display_off_mode( boolean off_mode )
{
	lcd_clear();
  lcd_const_string("OFF MODE");
  lcd_locate(0,1);
  if( off_mode ) {
	  lcd_const_string("PowrOFF ");
	} else {
	  lcd_const_string("PowrCONT");
  } 
}

boolean get_off_mode(void)
{
  boolean off_mode;	
  boolean off_mode_save;	
  unsigned char turn;     // CW,CCW

  off_mode = Read_b_eep( ADDR_OFF_MODE );
  off_mode_save = off_mode;
  while( true ) {
    turn = rotary_encoder();
    if( turn != ENCODER_STOP ) {  
      if( turn == CW ) {
	      off_mode = true;
      } else {  // CCW
	      off_mode = false;
      }
      display_off_mode( off_mode );
    }
    if( No_SW_ESC == hw_switch_test() ) {
	    Switch_enable = false;
      return off_mode_save;
	  }
	  if( No_SW_ENTER == hw_switch_test() ) {
		  Switch_enable = false;
		  return off_mode;
		}
  }	
}


//----------------------------------------------------------------------------
// ŜɊ֌W铮̐ݒ
//
// 1:㎥[h
// 2:[^[]~̓dON/OFF
//----------------------------------------------------------------------------
void set_gloval_action( void )
{
	enum type_driver_mode driver_mode;
  boolean               off_mode; 
	
	unsigned char parameter_no;
  enum type_encoder_direction turn;

  driver_mode = Read_b_eep( ADDR_DRIVER_MODE );
  off_mode    = Read_b_eep( ADDR_OFF_MODE );

  lcd_clear();
	parameter_no = 1;
	display_driver_mode( driver_mode );
  delay_ms(100);	
  while( true ) {
    turn = rotary_encoder();
    if( turn != ENCODER_STOP ) {
      if ( turn == CW  && parameter_no < 2 ) {
 	      parameter_no++;
      }
      if ( turn == CCW && parameter_no > 1 ) {
        parameter_no--;
      }
      switch( parameter_no ) {
	      case 1: display_driver_mode( driver_mode ); break;
	      case 2: display_off_mode( off_mode );       break;
	    }
    }
    if( No_SW_ENTER == hw_switch_test() ) {
	    Switch_enable = false;
	    LED3 = LED_ON;	    
      switch( parameter_no ) {
	      case 1: driver_mode = get_driver_mode();
	              Write_b_eep(ADDR_DRIVER_MODE, driver_mode);  
	              break;
        case 2: off_mode = get_off_mode();
                Write_b_eep(ADDR_OFF_MODE, off_mode);
                break;
      }
      LED3 = LED_OFF;
      delay_ms(500);
    }
    
    if( No_SW_ESC == hw_switch_test() ) {
	    Switch_enable = false;
	    delay_ms(500);
	    break;
	  }
  }
  display_sys_mode();
}


//----------------------------------------------------------------------------
// XCb`Ƃ̓̐ݒ
//----------------------------------------------------------------------------
void set_switch_action( unsigned char switch_no )
{
	unsigned char parameter_no;
  enum type_encoder_direction turn;

  lcd_clear();
  lcd_locate( 5, 0 );
  display_switch_no(switch_no);
 
  read_eep_action( switch_no );	
	parameter_no = 1;
	display_direction(Action.cwccw);
  delay_ms(100);	
  while( true ) {
    turn = rotary_encoder();
    if( turn != ENCODER_STOP ) {
      if ( turn == CW  && parameter_no != NUMBER_OF_PARAMETER ) {
 	      parameter_no++;
      }
      if ( turn == CCW && parameter_no != 1 ) {
        parameter_no--;
      }
      switch( parameter_no ) {
	      case 1: display_direction(Action.cwccw);    break;
	      case 2: display_unit(     Action.unit);     break;
	      case 3: display_interval( Action.interval); break;
	      case 4: display_step(     Action.step);     break;
	      case 5: display_priority( Action.priority); break;
	      case 6: display_torque(   Action.torque);   break;
	      case 7: display_next(     Action.next);     break;
	      case 8: display_repeat(   Action.repeat);   break;
	    }
    }
    if( No_SW_ENTER == hw_switch_test() ) {
	    Switch_enable = false;
	    LED3 = LED_ON;
	    read_eep_action( switch_no );
      switch( parameter_no ) {
	      case 1: Action.cwccw    = get_direction( Action.cwccw );   break;
        case 2: Action.unit     = get_unit( Action.unit);          break;
 	      case 3: Action.interval = get_interval( Action.interval, Action.unit ); break;
        case 4: Action.step     = get_step( Action.step );         break;
        case 5: Action.priority = get_priority( Action.priority ); break;
        case 6: Action.torque   = get_torque( Action.torque);      break;
 	      case 7: Action.next     = get_next( Action.next );         break;
 	      case 8: Action.repeat   = get_repeat( Action.repeat );     break;
      }
      write_eep_action( switch_no );
      LED3 = LED_OFF;
      delay_ms(500);
    }

    if( No_SW_ESC == hw_switch_test() ) {
	    Switch_enable = false;
	    display_switch_action( switch_no );
	    break;
	  }
  }
}

//----------------------------------------------------------------------------
// XCb`ꂽۂ̃eXg
//----------------------------------------------------------------------------
char switch_test( void )
{
	unsigned char sw_no;
	
	sw_no = -1;
	if( Motor_run == false ) {
		if( Action.next == 0 ) {
			if( Repeat_count > 1 ) {
				Repeat_count--;
				return Repeat_sw_no;
      } else {
        lcd_locate(7,0);
        lcd_const_string("0");  // Display Repeat_count == 0
      }  
    } else {
	    return Action.next;
	  }
	}

  sw_no =  hw_switch_test();
  if( sw_no != -1 ) {
	  Repeat_count = 0;
	}
    	
	return sw_no;
}


char hw_switch_test( void )
{
	char sw_no;

	sw_no = -1;		 
  if( Switch_enable == true ) {  
  	if( SW_OB1  == SW_ON ) { sw_no = No_SW_OB1;  }
	  if( SW_OB2  == SW_ON ) { sw_no = No_SW_OB2;  }
	  if( SW_OB3  == SW_ON ) { sw_no = No_SW_OB3;  }
  	if( SW_EXT1 == SW_ON ) { sw_no = No_SW_EXT1; }
	  if( SW_EXT2 == SW_ON ) { sw_no = No_SW_EXT2; }
  	if( SW_EXT3 == SW_ON ) { sw_no = No_SW_EXT3; }
  	if( SW_EXT4 == SW_ON ) { sw_no = No_SW_EXT4; }
  	if( SW_EXT5 == SW_ON ) { sw_no = No_SW_EXT5; }
  }
  if( sw_no != -1 ) {
    Switch_disable_timer = SW_DISABLE_TIME;
    Switch_enable = false;
  }
  
	return sw_no;
}


//----------------------------------------------------------------------------
// [^[hCo[nm
//----------------------------------------------------------------------------
void motor_driver_enable(void)
{
	MOTOR_ENABLE = 1;
	delay_ms(1);
	MOTOR_RESET = 1;
}
	

//----------------------------------------------------------------------------
// [^[hCo[nee
//----------------------------------------------------------------------------
void motor_driver_disable(void)
{
	MOTOR_ENABLE = 0;
	delay_ms(1);
	MOTOR_RESET = 0;
}


//----------------------------------------------------------------------------
// XCb`Ƃ̓
//----------------------------------------------------------------------------
void exec_action( short switch_no )
{
	long_union tmp;

	read_eep_action( switch_no );

 	if( Repeat_count == 0 ) {
		Repeat_count = Action.repeat;
		Repeat_sw_no = switch_no;
	} 
  Step_counter = Action.step;
	
  if( Action.priority >= Current_priority ) {
	  lcd_clear();
    display_switch_no( switch_no );
    display_repeat_run( Repeat_count );    
	  Current_priority = Action.priority;
	  
	  motor_driver_enable();

	  switch( Action.cwccw ) {
		  case M_HALT:
		    if( Off_mode ) {
	        Motor_power_off_timer = POWER_OFF_TIME; 
 	        Motor_power_off_wait = true;
			  }
			  break;
		  case M_CW:
		    MOTOR_CWCCW = 0;
		    Motor_power_off_wait = false;
		    break;
		  case M_CCW:
		    MOTOR_CWCCW = 1;
 		    Motor_power_off_wait = false;
		    break;
		}

    switch( Action.unit ) {
      case USEC:
        Timer1_prescale_bit1 = 1;  // prescale 1:4 
        Timer1_prescale_bit0 = 0;  // 3count = 1Microsec
        tmp.Lval = Action.interval * 3;
        CCPR1H = tmp.Bval[1];
        CCPR1L = tmp.Bval[0];
        CCP1_match_count = 1;
        break;
      case MSEC:
        Timer1_prescale_bit1 = 1;  // prescale 1:8 
        Timer1_prescale_bit0 = 1;  // 1500count = 1msec
        CCPR1H = 0x05;             // 1msecspecial event trigger   
        CCPR1L = 0xDC;
        CCP1_match_count = Action.interval;
        break;
      case SEC:
        Timer1_prescale_bit1 = 1;  // prescale 1:8 
        Timer1_prescale_bit0 = 1;  // 1500000count = 1msec
        CCPR1H = 0xEA;             // 40msecspecial event trigger   
        CCPR1L = 0x60;
        CCP1_match_count = Action.interval * 25;
        break;
    }

   
    switch( Action.torque ) {
	    case 100: MOTOR_TORQUE1 = 0; MOTOR_TORQUE2 = 0; break;
	    case  75: MOTOR_TORQUE1 = 1; MOTOR_TORQUE2 = 0; break;
	    case  50: MOTOR_TORQUE1 = 0; MOTOR_TORQUE2 = 1; break;
	    case  25: MOTOR_TORQUE1 = 1; MOTOR_TORQUE2 = 1; break;
    }
    
    Step_interval_counter = CCP1_match_count;
    TMR1H = 0;
    TMR1L = 0;    
    Timer1_enable = true;
    CCP1_interrupt_enable = true;
    Motor_run = true; 
	}
}

//----------------------------------------------------------------------------
// C[`
//----------------------------------------------------------------------------
void main()
{
  enum  type_encoder_direction turn;
	enum type_driver_mode driver_mode;
  short switch_no;
  char  buf[9];

  ADCON1bits.PCFG = 0b1111;  // digital I/O
  CMCONbits.CM    = 0b111;   // compatator off
  INTCON2bits.NOT_RBPU = 1;  // PortB pull-up disable

  SW_OB1_TRIS   = PORT_INPUT;
  SW_OB2_TRIS   = PORT_INPUT;
  SW_OB3_TRIS   = PORT_INPUT;
  SLIDE_SW_TRIS = PORT_INPUT;
  SW_EXT1_TRIS  = PORT_INPUT;
  SW_EXT2_TRIS  = PORT_INPUT;
  SW_EXT3_TRIS  = PORT_INPUT;
  SW_EXT4_TRIS  = PORT_INPUT;
  SW_EXT5_TRIS  = PORT_INPUT;

  LED1_TRIS     = PORT_OUTPUT;
  LED2_TRIS     = PORT_OUTPUT;
  LED3_TRIS     = PORT_OUTPUT;

  MOTOR_CLK_TRIS     = PORT_OUTPUT;
  MOTOR_RESET_TRIS   = PORT_OUTPUT;
  MOTOR_ENABLE_TRIS  = PORT_OUTPUT;
  MOTOR_MODE1_TRIS   = PORT_OUTPUT;
  MOTOR_MODE2_TRIS   = PORT_OUTPUT;
  MOTOR_CWCCW_TRIS   = PORT_OUTPUT;
  MOTOR_TORQUE1_TRIS = PORT_OUTPUT;
  MOTOR_TORQUE2_TRIS = PORT_OUTPUT;
  
  motor_driver_disable();

  LED1 = LED_OFF;
  LED2 = LED_OFF;
  LED3 = LED_OFF;

  rotary_encoder_init();
  lcd_init();

  //---- TMR0(16bitMODE) initialize --------------
  // Timer0_prescale_bit2 = 0   // PreScaler 1:2
  // Timer0_prescale_bit1 = 0   // PreScaler 1:2
  // Timer0_prescale_bit0 = 0   // PreScaler 1:2
/*
  Timer0_prescale_NOT_assign = true;  
  Timer0_mode_8bit           = false;  // 16bit mode
  Timer0_interrupt_enable    = false;
  Timer0_use_external_clock  = false;
  Timer0_enable              = true;
  Timer0_priority            = High_priority;
*/
  //---- TMR1(16bit) initialize -----------------
  Timer1_interrupt_enable    = false; 
  Timer1_use_external_clock  = false;
  Timer1_enable              = false;
  Timer1_priority            = High_priority;

  //---- TMR2 initialize ---------------------
  Timer2_interrupt_enable    = false; 
  Timer2_priority            = Low_priority;
  Timer2_postscale_bit3      = 0;  // postscale 1:1
  Timer2_postscale_bit2      = 0;
  Timer2_postscale_bit1      = 0;
  Timer2_postscale_bit0      = 0;
  Timer2_prescale_bit1       = 0;  // prescale 1:4
  Timer2_prescale_bit0       = 1;
  Timer2_enable              = true;
  PR2 = 180; // 60usec
//  PR2 = 0; // 60usec
    
  //---- TMR3(16bitMODE) initialize -----------------
  Timer3_interrupt_enable    = true; 
  Timer3_prescale_bit0       = 0;
  Timer3_prescale_bit1       = 0;
  Timer3_mode_16bit          = true;
  Timer3_use_external_clock  = false;
  Timer3_not_sync_ext_clock  = true;
  Timer3_priority            = Low_priority;
  Timer3_enable              = true;
  TMR3H = 0xD1;   // 1ms
  TMR3L = 0x20;   // 1ms

  //---- CCP module initialize -----------------
  CCP1_mode_bit3        = 1; // Compare mode: trigger special event, reset timer, CCPxIF bit is set
  CCP1_mode_bit2        = 0;
  CCP1_mode_bit1        = 1;
  CCP1_mode_bit0        = 1;
  CCP1_interrupt_enable = false; 
  CCP1_priority         = High_priority;
  
  delay_ms(100);

  lcd_clear();
  lcd_const_string("Program");
  lcd_locate(0,1);
  lcd_const_string( VERSION );
  delay_ms(1000);

  //---- XChXCb`ɐݒ肷ƃZbgAbv[hɂȂ ---------------------
  switch_no = 0;
  if( SLIDE_SW == SW_ON ) {
    RCONbits.IPEN   = 1;          // 1:enable 0:disable priority levels
    INTCONbits.GIEH = 1;          // 1:enable 0:disable interrupts
    INTCONbits.GIEL = 1;          // 1:enable 0:disable interrupts
    display_sys_mode();
    while( true ) {
      turn = rotary_encoder();
      if( turn != ENCODER_STOP ) {
        if ( turn == CW ) {
	        if( switch_no < NUMBER_OF_SW ) {
 	          switch_no++;
 	        } 
        }
        if ( turn == CCW ) {
	        if( switch_no > 0 ) {
            switch_no--;
          } 
        }
        if( switch_no == 0 ) {
	        display_sys_mode();
        } else {
          display_switch_action( switch_no );
        }        
      }

	    if( No_SW_ENTER ==  hw_switch_test() ) {
		    Switch_enable = false;
		    if( switch_no == 0 ) {
			    set_gloval_action();
			  } else {
  		    set_switch_action( switch_no );
  		  } 
		  }
    } // XCb`Oɂēd꒼ȂƐݒ胂[h͔Ȃ
  }

  //---- 荞݋ ---------------------------------------------
  RCONbits.IPEN   = 1;          // 1:enable 0:disable priority levels
  INTCONbits.GIEH = 1;          // 1:enable 0:disable interrupts
  INTCONbits.GIEL = 1;          // 1:enable 0:disable interrupts

  //--------- [^[䃂[h\ -----------------------------------
  Off_mode = Read_b_eep( ADDR_OFF_MODE );
  driver_mode = Read_b_eep( ADDR_DRIVER_MODE );
  switch( driver_mode ) {
    case x1:  MOTOR_MODE1 = 0; MOTOR_MODE2 = 0; break;
    case x2:  MOTOR_MODE1 = 1; MOTOR_MODE2 = 0; break;
    case x8:  MOTOR_MODE1 = 1; MOTOR_MODE2 = 1; break;
    case x16: MOTOR_MODE1 = 0; MOTOR_MODE2 = 1; break;
  }

  //--------- [^[ -----------------------------------
  Motor_run = false;

  Current_priority = 1;
  lcd_clear();
  lcd_const_string("WAIT");
  Action.step   = 0;
  Action.next   = 0;
  Repeat_count  = 0;
  Step_counter  = 0;
  Switch_enable = true;
  Motor_power_off_wait = false;
  while( true ) {
    if( (switch_no = switch_test()) != -1 ) {
	    Switch_enable = false;
      LED1 = LED_OFF;
      LED2 = LED_OFF;
      LED3 = LED_OFF;
	    switch( switch_no ) {
        case No_SW_OB1: LED1 = LED_ON; break;
        case No_SW_OB2: LED2 = LED_ON; break;
        case No_SW_OB3: LED3 = LED_ON; break;
      } 
      exec_action( switch_no );
	  }
	  display_step_num( Step_counter );
	  
	  if( Motor_run == false ) {
		  LED1 = LED_OFF;
      LED2 = LED_OFF;
      LED3 = LED_OFF;
      lcd_home();
      lcd_const_string("WAIT");
    }
    
    if( Off_mode == true ) {
      if( (Motor_power_off_wait == true) && (Motor_power_off_timer == 0) ) {
	      motor_driver_disable();
	      Motor_power_off_wait = false;
	    }
	  }
/*	  
	  if(Switch_enable == true) {
		  LED3 = LED_ON;
		} else {
		  LED3 = LED_OFF;
    }

	  if( Switch_disable_timer == 0) {
		  LED2 = LED_ON;
		} else {
		  LED2 = LED_OFF;
    }
*/   
	
  }
}

//================================================================= E N D =========
